from numpy.typing import NDArray
from typing import Any, ClassVar, Dict, Iterator, List, Optional, Tuple, Union

from typing import overload
import numpy
import pytket._tket.architecture
import pytket._tket.pauli
import pytket._tket.unit_id
import sympy

class BarrierOp(Op):
    def __init__(self, signature: List[EdgeType], data: str) -> None: ...
    @property
    def data(self) -> str: ...

class BasisOrder:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    dlo: ClassVar[BasisOrder] = ...
    ilo: ClassVar[BasisOrder] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class CXConfigType:
    __members__: ClassVar[dict] = ...  # read-only
    MultiQGate: ClassVar[CXConfigType] = ...
    Snake: ClassVar[CXConfigType] = ...
    Star: ClassVar[CXConfigType] = ...
    Tree: ClassVar[CXConfigType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class CircBox(Op):
    def __init__(self, circ: Circuit) -> None: ...
    def get_circuit(self) -> Circuit: ...

class Circuit:
    add_wasm: ClassVar[function] = ...
    add_wasm_to_reg: ClassVar[function] = ...
    name: Optional[str]
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, name: str) -> None: ...
    @overload
    def __init__(self, n_qubits: int, name: Optional[str] = ...) -> None: ...
    @overload
    def __init__(self, n_qubits: int, n_bits: int, name: Optional[str] = ...) -> None: ...
    @overload
    def CCX(self, control_0: int, control_1: int, target: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CCX(self, control_0: pytket._tket.unit_id.Qubit, control_1: pytket._tket.unit_id.Qubit, target: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CH(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CH(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CRx(self, angle: Union[sympy.Expr,float], control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CRx(self, angle: Union[sympy.Expr,float], control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CRy(self, angle: Union[sympy.Expr,float], control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CRy(self, angle: Union[sympy.Expr,float], control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CRz(self, angle: Union[sympy.Expr,float], control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CRz(self, angle: Union[sympy.Expr,float], control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CSWAP(self, control: int, target_0: int, target_1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CSWAP(self, control: pytket._tket.unit_id.Qubit, target_0: pytket._tket.unit_id.Qubit, target_1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CSX(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CSX(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CSXdg(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CSXdg(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CU1(self, angle: Union[sympy.Expr,float], control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CU1(self, angle: Union[sympy.Expr,float], control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CU3(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CU3(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CV(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CV(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CVdg(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CVdg(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CX(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CX(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CY(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CY(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def CZ(self, control_qubit: int, target_qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def CZ(self, control_qubit: pytket._tket.unit_id.Qubit, target_qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def ECR(self, qubit_0: int, qubit_1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def ECR(self, qubit_0: pytket._tket.unit_id.Qubit, qubit_1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def ESWAP(self, angle: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def ESWAP(self, angle: Union[sympy.Expr,float], qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def FSim(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def FSim(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def H(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def H(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def ISWAP(self, angle: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def ISWAP(self, angle: Union[sympy.Expr,float], qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def ISWAPMax(self, qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def ISWAPMax(self, qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Measure(self, qubit: int, bit_index: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Measure(self, qubit: pytket._tket.unit_id.Qubit, bit: pytket._tket.unit_id.Bit, **kwargs: Any) -> Circuit: ...
    def Phase(self, arg0: Union[sympy.Expr,float], **kwargs: Any) -> Circuit: ...
    @overload
    def PhasedISWAP(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def PhasedISWAP(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def PhasedX(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def PhasedX(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Rx(self, angle: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Rx(self, angle: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Ry(self, angle: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Ry(self, angle: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Rz(self, angle: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Rz(self, angle: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def S(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def S(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def SWAP(self, qubit_0: int, qubit_1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def SWAP(self, qubit_0: pytket._tket.unit_id.Qubit, qubit_1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def SX(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def SX(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def SXdg(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def SXdg(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Sdg(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Sdg(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Sycamore(self, qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Sycamore(self, qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def T(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def T(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def TK1(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def TK1(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def TK2(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def TK2(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Tdg(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Tdg(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def U1(self, angle: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def U1(self, angle: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def U2(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def U2(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def U3(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def U3(self, angle0: Union[sympy.Expr,float], angle1: Union[sympy.Expr,float], angle2: Union[sympy.Expr,float], qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def V(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def V(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Vdg(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Vdg(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def X(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def X(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def XXPhase(self, angle: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def XXPhase(self, qubit0: Union[sympy.Expr,float], qubit1: pytket._tket.unit_id.Qubit, angle: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def XXPhase3(self, angle: Union[sympy.Expr,float], qubit0: int, qubit1: int, qubit2: int, **kwargs: Any) -> Circuit: ...
    @overload
    def XXPhase3(self, angle: Union[sympy.Expr,float], qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, qubit2: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Y(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Y(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def YYPhase(self, angle: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def YYPhase(self, qubit0: Union[sympy.Expr,float], qubit1: pytket._tket.unit_id.Qubit, angle: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def Z(self, qubit: int, **kwargs: Any) -> Circuit: ...
    @overload
    def Z(self, qubit: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def ZZMax(self, qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def ZZMax(self, qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def ZZPhase(self, angle: Union[sympy.Expr,float], qubit0: int, qubit1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def ZZPhase(self, angle: Union[sympy.Expr,float], qubit0: pytket._tket.unit_id.Qubit, qubit1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    def _add_w_register(self, size: int) -> None: ...
    @overload
    def _add_wasm(self, funcname: str, wasm_uid: str, width_i_parameter: List[int], width_o_parameter: List[int], args: List[int], wasm_wire_args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def _add_wasm(self, funcname: str, wasm_uid: str, width_i_parameter: List[int], width_o_parameter: List[int], args: List[pytket._tket.unit_id.Bit], wasm_wire_args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def _add_wasm(self, funcname: str, wasm_uid: str, list_reg_in: List[pytket._tket.unit_id.BitRegister], list_reg_out: List[pytket._tket.unit_id.BitRegister], wasm_wire_args: List[int], **kwargs: Any) -> Circuit: ...
    def _classical_eval(self, arg0: Dict[pytket._tket.unit_id.Bit,bool]) -> Dict[pytket._tket.unit_id.Bit,bool]: ...
    def _n_vertices(self) -> int: ...
    def _to_graphviz_file(self, filename: str) -> None: ...
    @overload
    def add_assertion(self, box: ProjectorAssertionBox, qubits: List[int], ancilla: Optional[int] = ..., name: Optional[str] = ...) -> Circuit: ...
    @overload
    def add_assertion(self, box: ProjectorAssertionBox, qubits: List[pytket._tket.unit_id.Qubit], ancilla: Optional[pytket._tket.unit_id.Qubit] = ..., name: Optional[str] = ...) -> Circuit: ...
    @overload
    def add_assertion(self, box: StabiliserAssertionBox, qubits: List[int], ancilla: int, name: Optional[str] = ...) -> Circuit: ...
    @overload
    def add_assertion(self, box: StabiliserAssertionBox, qubits: List[pytket._tket.unit_id.Qubit], ancilla: pytket._tket.unit_id.Qubit, name: Optional[str] = ...) -> Circuit: ...
    @overload
    def add_barrier(self, qubits: List[int], bits: List[int] = ..., data: str = ...) -> Circuit: ...
    @overload
    def add_barrier(self, units: List[pytket._tket.unit_id.UnitID], data: str = ...) -> Circuit: ...
    def add_bit(self, id: pytket._tket.unit_id.Bit, reject_dups: bool = ...) -> None: ...
    def add_blank_wires(self, number: int) -> None: ...
    @overload
    def add_c_and(self, arg0_in: int, arg1_in: int, arg_out: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_and(self, arg0_in: pytket._tket.unit_id.Bit, arg1_in: pytket._tket.unit_id.Bit, arg_out: pytket._tket.unit_id.Bit, **kwargs: Any) -> Circuit: ...
    def add_c_and_to_registers(self, reg0_in: pytket._tket.unit_id.BitRegister, reg1_in: pytket._tket.unit_id.BitRegister, reg_out: pytket._tket.unit_id.BitRegister, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_copybits(self, args_in: List[int], args_out: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_copybits(self, args_in: List[pytket._tket.unit_id.Bit], args_out: List[pytket._tket.unit_id.Bit], **kwargs: Any) -> Circuit: ...
    def add_c_copyreg(self, input_reg: pytket._tket.unit_id.BitRegister, output_reg: pytket._tket.unit_id.BitRegister, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_modifier(self, values: List[bool], args_in: List[int], arg_inout: int, name: str = ..., **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_modifier(self, values: List[bool], args_in: List[pytket._tket.unit_id.Bit], arg_inout: pytket._tket.unit_id.Bit, name: str = ..., **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_not(self, arg_in: int, arg_out: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_not(self, arg_in: pytket._tket.unit_id.Bit, arg_out: pytket._tket.unit_id.Bit, **kwargs: Any) -> Circuit: ...
    def add_c_not_to_registers(self, reg_in: pytket._tket.unit_id.BitRegister, reg_out: pytket._tket.unit_id.BitRegister, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_or(self, arg0_in: int, arg1_in: int, arg_out: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_or(self, arg0_in: pytket._tket.unit_id.Bit, arg1_in: pytket._tket.unit_id.Bit, arg_out: pytket._tket.unit_id.Bit, **kwargs: Any) -> Circuit: ...
    def add_c_or_to_registers(self, reg0_in: pytket._tket.unit_id.BitRegister, reg1_in: pytket._tket.unit_id.BitRegister, reg_out: pytket._tket.unit_id.BitRegister, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_predicate(self, values: List[bool], args_in: List[int], arg_out: int, name: str = ..., **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_predicate(self, values: List[bool], args_in: List[pytket._tket.unit_id.Bit], arg_out: pytket._tket.unit_id.Bit, name: str = ..., **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_range_predicate(self, minval: int, maxval: int, args_in: List[int], arg_out: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_range_predicate(self, minval: int, maxval: int, args_in: List[pytket._tket.unit_id.Bit], arg_out: pytket._tket.unit_id.Bit, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_register(self, name: str, size: int) -> pytket._tket.unit_id.BitRegister: ...
    @overload
    def add_c_register(self, register: pytket._tket.unit_id.BitRegister) -> pytket._tket.unit_id.BitRegister: ...
    @overload
    def add_c_setbits(self, values: List[bool], args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_setbits(self, values: List[bool], args: List[pytket._tket.unit_id.Bit], **kwargs: Any) -> Circuit: ...
    def add_c_setreg(self, value: int, arg: pytket._tket.unit_id.BitRegister, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_transform(self, values: List[int], args: List[int], name: str = ..., **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_transform(self, values: List[int], args: List[pytket._tket.unit_id.Bit], name: str = ..., **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_xor(self, arg0_in: int, arg1_in: int, arg_out: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_c_xor(self, arg0_in: pytket._tket.unit_id.Bit, arg1_in: pytket._tket.unit_id.Bit, arg_out: pytket._tket.unit_id.Bit, **kwargs: Any) -> Circuit: ...
    def add_c_xor_to_registers(self, reg0_in: pytket._tket.unit_id.BitRegister, reg1_in: pytket._tket.unit_id.BitRegister, reg_out: pytket._tket.unit_id.BitRegister, **kwargs: Any) -> Circuit: ...
    @overload
    def add_circbox(self, circbox: CircBox, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_circbox(self, circbox: CircBox, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_circuit(self, circuit: Circuit, qubits: List[pytket._tket.unit_id.Qubit], bits: List[pytket._tket.unit_id.Bit] = ...) -> Circuit: ...
    @overload
    def add_circuit(self, circuit: Circuit, qubits: List[int], bits: List[int] = ...) -> Circuit: ...
    def add_classicalexpbox_bit(self, expression: object, target: List[pytket._tket.unit_id.Bit], **kwargs: Any) -> Circuit: ...
    def add_classicalexpbox_register(self, expression: object, target: List[pytket._tket.unit_id.Bit], **kwargs: Any) -> Circuit: ...
    @overload
    def add_conditional_barrier(self, barrier_qubits: List[int], barrier_bits: List[int], condition_bits: List[int], value: int, data: str = ...) -> Circuit: ...
    @overload
    def add_conditional_barrier(self, barrier_args: List[pytket._tket.unit_id.UnitID], condition_bits: List[pytket._tket.unit_id.Bit], value: int, data: str = ...) -> Circuit: ...
    @overload
    def add_conjugation_box(self, box: ConjugationBox, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_conjugation_box(self, box: ConjugationBox, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_custom_gate(self, definition: CustomGateDef, params: List[Union[sympy.Expr,float]], qubits: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_custom_gate(self, definition: CustomGateDef, params: List[Union[sympy.Expr,float]], qubits: List[pytket._tket.unit_id.Qubit], **kwargs: Any) -> Circuit: ...
    @overload
    def add_diagonal_box(self, box: DiagonalBox, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_diagonal_box(self, box: DiagonalBox, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_expbox(self, expbox: ExpBox, qubit_0: int, qubit_1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_expbox(self, expbox: ExpBox, qubit_0: pytket._tket.unit_id.Qubit, qubit_1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, Op: Op, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, Op: Op, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, type: OpType, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, type: OpType, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, type: OpType, angle: Union[sympy.Expr,float], args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, type: OpType, angle: Union[sympy.Expr,float], args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, type: OpType, angles: List[Union[sympy.Expr,float]], args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_gate(self, type: OpType, angles: List[Union[sympy.Expr,float]], args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexed_tensored_u2(self, box: MultiplexedTensoredU2Box, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexed_tensored_u2(self, box: MultiplexedTensoredU2Box, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexedrotation(self, box: MultiplexedRotationBox, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexedrotation(self, box: MultiplexedRotationBox, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexedu2(self, box: MultiplexedU2Box, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexedu2(self, box: MultiplexedU2Box, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexor(self, box: MultiplexorBox, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_multiplexor(self, box: MultiplexorBox, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_pauliexpbox(self, pauliexpbox: PauliExpBox, qubits: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_pauliexpbox(self, pauliexpbox: PauliExpBox, qubits: List[pytket._tket.unit_id.Qubit], **kwargs: Any) -> Circuit: ...
    @overload
    def add_pauliexpcommutingsetbox(self, pauliexpcommutingsetbox: PauliExpCommutingSetBox, qubits: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_pauliexpcommutingsetbox(self, pauliexpcommutingsetbox: PauliExpCommutingSetBox, qubits: List[pytket._tket.unit_id.Qubit], **kwargs: Any) -> Circuit: ...
    @overload
    def add_pauliexppairbox(self, pauliexppairbox: PauliExpPairBox, qubits: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_pauliexppairbox(self, pauliexppairbox: PauliExpPairBox, qubits: List[pytket._tket.unit_id.Qubit], **kwargs: Any) -> Circuit: ...
    def add_phase(self, a: Union[sympy.Expr,float]) -> Circuit: ...
    @overload
    def add_phasepolybox(self, phasepolybox: PhasePolyBox, qubits: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_phasepolybox(self, phasepolybox: PhasePolyBox, qubits: List[pytket._tket.unit_id.Qubit], **kwargs: Any) -> Circuit: ...
    @overload
    def add_q_register(self, name: str, size: int) -> pytket._tket.unit_id.QubitRegister: ...
    @overload
    def add_q_register(self, register: pytket._tket.unit_id.QubitRegister) -> pytket._tket.unit_id.QubitRegister: ...
    @overload
    def add_qcontrolbox(self, qcontrolbox: QControlBox, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_qcontrolbox(self, qcontrolbox: QControlBox, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    def add_qubit(self, id: pytket._tket.unit_id.Qubit, reject_dups: bool = ...) -> None: ...
    @overload
    def add_state_preparation_box(self, box: StatePreparationBox, args: List[pytket._tket.unit_id.UnitID], **kwargs: Any) -> Circuit: ...
    @overload
    def add_state_preparation_box(self, box: StatePreparationBox, args: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_toffolibox(self, toffolibox: ToffoliBox, qubits: List[int], **kwargs: Any) -> Circuit: ...
    @overload
    def add_toffolibox(self, toffolibox: ToffoliBox, qubits: List[pytket._tket.unit_id.Qubit], **kwargs: Any) -> Circuit: ...
    @overload
    def add_unitary1qbox(self, unitarybox: Unitary1qBox, qubit_0: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_unitary1qbox(self, unitarybox: Unitary1qBox, qubit_0: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def add_unitary2qbox(self, unitarybox: Unitary2qBox, qubit_0: int, qubit_1: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_unitary2qbox(self, unitarybox: Unitary2qBox, qubit_0: pytket._tket.unit_id.Qubit, qubit_1: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    @overload
    def add_unitary3qbox(self, unitarybox: Unitary3qBox, qubit_0: int, qubit_1: int, qubit_2: int, **kwargs: Any) -> Circuit: ...
    @overload
    def add_unitary3qbox(self, unitarybox: Unitary3qBox, qubit_0: pytket._tket.unit_id.Qubit, qubit_1: pytket._tket.unit_id.Qubit, qubit_2: pytket._tket.unit_id.Qubit, **kwargs: Any) -> Circuit: ...
    def append(self, circuit: Circuit) -> None: ...
    def commands_of_type(self, optype: OpType) -> List[Command]: ...
    def copy(self) -> Circuit: ...
    def dagger(self) -> Circuit: ...
    def depth(self) -> int: ...
    def depth_2q(self) -> int: ...
    @overload
    def depth_by_type(self, type: OpType) -> int: ...
    @overload
    def depth_by_type(self, types: set[OpType]) -> int: ...
    def flatten_registers(self) -> Dict[pytket._tket.unit_id.UnitID,pytket._tket.unit_id.UnitID]: ...
    def free_symbols(self) -> set[sympy.Symbol]: ...
    @classmethod
    def from_dict(cls, arg0: dict) -> Circuit: ...
    def get_c_register(self, name: str) -> pytket._tket.unit_id.BitRegister: ...
    def get_commands(self) -> List[Command]: ...
    def get_q_register(self, name: str) -> pytket._tket.unit_id.QubitRegister: ...
    def get_statevector(self) -> NDArray[numpy.complex128]: ...
    def get_unitary(self) -> NDArray[numpy.complex128]: ...
    def get_unitary_times_other(self, matr: NDArray[numpy.complex128]) -> NDArray[numpy.complex128]: ...
    def implicit_qubit_permutation(self) -> Dict[pytket._tket.unit_id.Qubit,pytket._tket.unit_id.Qubit]: ...
    def is_symbolic(self) -> bool: ...
    def measure_all(self) -> Circuit: ...
    def measure_register(self, arg0: pytket._tket.unit_id.QubitRegister, arg1: str) -> Circuit: ...
    def n_1qb_gates(self) -> int: ...
    def n_2qb_gates(self) -> int: ...
    def n_gates_of_type(self, type: OpType) -> int: ...
    def n_nqb_gates(self, size: int) -> int: ...
    def ops_of_type(self, optype: OpType) -> List[Op]: ...
    def qubit_create(self, arg0: pytket._tket.unit_id.Qubit) -> None: ...
    def qubit_create_all(self) -> None: ...
    def qubit_discard(self, arg0: pytket._tket.unit_id.Qubit) -> None: ...
    def qubit_discard_all(self) -> None: ...
    def qubit_is_created(self, arg0: pytket._tket.unit_id.Qubit) -> bool: ...
    def qubit_is_discarded(self, arg0: pytket._tket.unit_id.Qubit) -> bool: ...
    def remove_blank_wires(self) -> None: ...
    def rename_units(self, map: Dict[pytket._tket.unit_id.UnitID,pytket._tket.unit_id.UnitID]) -> bool: ...
    def replace_SWAPs(self) -> None: ...
    def replace_implicit_wire_swaps(self) -> None: ...
    @overload
    def substitute_named(self, op: Op, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, repl: Circuit, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: CircBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: Unitary1qBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: Unitary2qBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: Unitary3qBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: ExpBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: PauliExpBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: ToffoliBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: QControlBox, opgroup: str) -> bool: ...
    @overload
    def substitute_named(self, box: CustomGate, opgroup: str) -> bool: ...
    @overload
    def symbol_substitution(self, symbol_map: Dict[sympy.Symbol,Union[sympy.Expr,float]]) -> None: ...
    @overload
    def symbol_substitution(self, symbol_map: Dict[sympy.Symbol,float]) -> None: ...
    def to_dict(self) -> dict: ...
    def to_latex_file(self, filename: str) -> None: ...
    def transpose(self) -> Circuit: ...
    def valid_connectivity(self, arch: pytket._tket.architecture.Architecture, directed: bool, allow_bridge: bool = ...) -> bool: ...
    def __eq__(self, arg0: object) -> bool: ...
    def __getstate__(self) -> tuple: ...
    def __hash__(self) -> int: ...
    def __iter__(self) -> Iterator: ...
    def __mul__(self, arg0: Circuit) -> Circuit: ...
    def __rshift__(self, arg0: Circuit) -> Circuit: ...
    def __setstate__(self, arg0: tuple) -> None: ...
    @property
    def _dag_data(self) -> Tuple[set[int],set[int],set[int],set[int],set[int],set[int],Dict[int,str],Dict[int,str],Dict[int,str],set[Tuple[int,int,int,int,str]]]: ...
    @property
    def bit_readout(self) -> Dict[pytket._tket.unit_id.Bit,int]: ...
    @property
    def bits(self) -> List[pytket._tket.unit_id.Bit]: ...
    @property
    def c_registers(self) -> List[pytket._tket.unit_id.BitRegister]: ...
    @property
    def created_qubits(self) -> List[pytket._tket.unit_id.Qubit]: ...
    @property
    def discarded_qubits(self) -> List[pytket._tket.unit_id.Qubit]: ...
    @property
    def is_simple(self) -> bool: ...
    @property
    def n_bits(self) -> int: ...
    @property
    def n_gates(self) -> int: ...
    @property
    def n_qubits(self) -> int: ...
    @property
    def opgroups(self) -> set[str]: ...
    @property
    def phase(self) -> Union[sympy.Expr,float]: ...
    @property
    def q_registers(self) -> List[pytket._tket.unit_id.QubitRegister]: ...
    @property
    def qubit_readout(self) -> Dict[pytket._tket.unit_id.Qubit,int]: ...
    @property
    def qubit_to_bit_map(self) -> Dict[pytket._tket.unit_id.Qubit,pytket._tket.unit_id.Bit]: ...
    @property
    def qubits(self) -> List[pytket._tket.unit_id.Qubit]: ...

class ClassicalEvalOp(ClassicalOp):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class ClassicalExpBox(Op):
    def __init__(self, n_i: int, n_io: int, n_o: int, exp: object) -> None: ...
    def content_equality(self, arg0: ClassicalExpBox) -> bool: ...
    def get_exp(self) -> object: ...
    def get_n_i(self) -> int: ...
    def get_n_io(self) -> int: ...
    def get_n_o(self) -> int: ...

class ClassicalOp(Op):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @property
    def n_input_outputs(self) -> int: ...
    @property
    def n_inputs(self) -> int: ...
    @property
    def n_outputs(self) -> int: ...

class Command:
    def __init__(self, op: Op, args: List[pytket._tket.unit_id.UnitID]) -> None: ...
    def free_symbols(self) -> set[sympy.Symbol]: ...
    def __eq__(self, arg0: object) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def args(self) -> List[pytket._tket.unit_id.UnitID]: ...
    @property
    def bits(self) -> List[pytket._tket.unit_id.Bit]: ...
    @property
    def op(self) -> Op: ...
    @property
    def opgroup(self) -> Optional[str]: ...
    @property
    def qubits(self) -> List[pytket._tket.unit_id.Qubit]: ...

class Conditional(Op):
    def __init__(self, op: Op, width: int, value: int) -> None: ...
    @property
    def op(self) -> Op: ...
    @property
    def value(self) -> int: ...
    @property
    def width(self) -> int: ...

class ConjugationBox(Op):
    def __init__(self, compute: Op, action: Op, uncompute: Optional[Op] = ...) -> None: ...
    def get_action(self) -> Op: ...
    def get_circuit(self) -> Circuit: ...
    def get_compute(self) -> Op: ...
    def get_uncompute(self) -> Optional[Op]: ...

class CopyBitsOp(ClassicalEvalOp):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class CustomGate(Op):
    def __init__(self, gatedef: CustomGateDef, params: List[Union[sympy.Expr,float]]) -> None: ...
    def get_circuit(self) -> Circuit: ...
    @property
    def gate(self) -> CustomGateDef: ...
    @property
    def name(self) -> str: ...
    @property
    def params(self) -> List[Union[sympy.Expr,float]]: ...

class CustomGateDef:
    def __init__(self, arg0: str, arg1: Circuit, arg2: List[sympy.Symbol]) -> None: ...
    @classmethod
    def define(cls, name: str, circ: Circuit, args: List[sympy.Symbol]) -> CustomGateDef: ...
    @classmethod
    def from_dict(cls, arg0: dict) -> CustomGateDef: ...
    def to_dict(self) -> dict: ...
    @property
    def args(self) -> List[sympy.Symbol]: ...
    @property
    def arity(self) -> int: ...
    @property
    def definition(self) -> Circuit: ...
    @property
    def name(self) -> str: ...

class DiagonalBox(Op):
    def __init__(self, diagonal: NDArray[numpy.complex128], upper_triangle: bool = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_diagonal(self) -> NDArray[numpy.complex128]: ...
    def is_upper_triangle(self) -> bool: ...

class EdgeType:
    __members__: ClassVar[dict] = ...  # read-only
    Boolean: ClassVar[EdgeType] = ...
    Classical: ClassVar[EdgeType] = ...
    Quantum: ClassVar[EdgeType] = ...
    WASM: ClassVar[EdgeType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class ExpBox(Op):
    def __init__(self, A: NDArray[numpy.complex128], t: float, basis: BasisOrder = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...

class MetaOp(Op):
    def __init__(self, type: OpType, signature: List[EdgeType], data: str) -> None: ...
    @property
    def data(self) -> str: ...

class MultiBitOp(ClassicalEvalOp):
    def __init__(self, op: ClassicalEvalOp, multiplier: int) -> None: ...
    @property
    def basic_op(self) -> ClassicalEvalOp: ...

class MultiplexedRotationBox(Op):
    @overload
    def __init__(self, bistring_to_op_list: List[Tuple[List[bool],Op]]) -> None: ...
    @overload
    def __init__(self, op_map: Dict[List[bool],Op]) -> None: ...
    @overload
    def __init__(self, angles: List[float], axis: OpType) -> None: ...
    def get_bitstring_op_pair_list(self) -> List[Tuple[List[bool],Op]]: ...
    def get_circuit(self) -> Circuit: ...
    def get_op_map(self) -> Dict[tuple,Op]: ...

class MultiplexedTensoredU2Box(Op):
    @overload
    def __init__(self, bistring_to_op_list: List[Tuple[List[bool],List[Op]]]) -> None: ...
    @overload
    def __init__(self, op_map: Dict[List[bool],List[Op]]) -> None: ...
    def get_bitstring_op_pair_list(self) -> List[Tuple[List[bool],List[Op]]]: ...
    def get_circuit(self) -> Circuit: ...
    def get_op_map(self) -> Dict[tuple,List[Op]]: ...

class MultiplexedU2Box(Op):
    @overload
    def __init__(self, bistring_to_op_list: List[Tuple[List[bool],Op]], impl_diag: bool = ...) -> None: ...
    @overload
    def __init__(self, op_map: Dict[List[bool],Op], impl_diag: bool = ...) -> None: ...
    def get_bitstring_op_pair_list(self) -> List[Tuple[List[bool],Op]]: ...
    def get_circuit(self) -> Circuit: ...
    def get_impl_diag(self) -> bool: ...
    def get_op_map(self) -> Dict[tuple,Op]: ...

class MultiplexorBox(Op):
    @overload
    def __init__(self, bistring_to_op_list: List[Tuple[List[bool],Op]]) -> None: ...
    @overload
    def __init__(self, op_map: Dict[List[bool],Op]) -> None: ...
    def get_bitstring_op_pair_list(self) -> List[Tuple[List[bool],Op]]: ...
    def get_circuit(self) -> Circuit: ...
    def get_op_map(self) -> Dict[tuple,Op]: ...

class Op:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @overload
    @classmethod
    def create(cls, arg0: OpType) -> Op: ...
    @overload
    @classmethod
    def create(cls, arg0: OpType, arg1: Union[sympy.Expr,float]) -> Op: ...
    @overload
    @classmethod
    def create(cls, arg0: OpType, arg1: List[Union[sympy.Expr,float]]) -> Op: ...
    def free_symbols(self) -> set[sympy.Symbol]: ...
    def get_name(self, latex: bool = ...) -> str: ...
    def get_unitary(self) -> NDArray[numpy.complex128]: ...
    def is_clifford(self) -> bool: ...
    def is_clifford_type(self) -> bool: ...
    def is_gate(self) -> bool: ...
    def __eq__(self, arg0: object) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def dagger(self) -> Op: ...
    @property
    def n_qubits(self) -> int: ...
    @property
    def params(self) -> List[Union[sympy.Expr,float]]: ...
    @property
    def transpose(self) -> Op: ...
    @property
    def type(self) -> OpType: ...

class OpType:
    __members__: ClassVar[dict] = ...  # read-only
    BRIDGE: ClassVar[OpType] = ...
    Barrier: ClassVar[OpType] = ...
    Branch: ClassVar[OpType] = ...
    CCX: ClassVar[OpType] = ...
    CH: ClassVar[OpType] = ...
    CRx: ClassVar[OpType] = ...
    CRy: ClassVar[OpType] = ...
    CRz: ClassVar[OpType] = ...
    CSWAP: ClassVar[OpType] = ...
    CSX: ClassVar[OpType] = ...
    CSXdg: ClassVar[OpType] = ...
    CU1: ClassVar[OpType] = ...
    CU3: ClassVar[OpType] = ...
    CV: ClassVar[OpType] = ...
    CVdg: ClassVar[OpType] = ...
    CX: ClassVar[OpType] = ...
    CY: ClassVar[OpType] = ...
    CZ: ClassVar[OpType] = ...
    CircBox: ClassVar[OpType] = ...
    ClassicalExpBox: ClassVar[OpType] = ...
    ClassicalTransform: ClassVar[OpType] = ...
    CnRy: ClassVar[OpType] = ...
    CnX: ClassVar[OpType] = ...
    CnY: ClassVar[OpType] = ...
    CnZ: ClassVar[OpType] = ...
    Conditional: ClassVar[OpType] = ...
    CopyBits: ClassVar[OpType] = ...
    CustomGate: ClassVar[OpType] = ...
    DiagonalBox: ClassVar[OpType] = ...
    ECR: ClassVar[OpType] = ...
    ESWAP: ClassVar[OpType] = ...
    ExpBox: ClassVar[OpType] = ...
    ExplicitModifier: ClassVar[OpType] = ...
    ExplicitPredicate: ClassVar[OpType] = ...
    FSim: ClassVar[OpType] = ...
    Goto: ClassVar[OpType] = ...
    H: ClassVar[OpType] = ...
    ISWAP: ClassVar[OpType] = ...
    ISWAPMax: ClassVar[OpType] = ...
    Label: ClassVar[OpType] = ...
    Measure: ClassVar[OpType] = ...
    MultiBit: ClassVar[OpType] = ...
    MultiplexedRotationBox: ClassVar[OpType] = ...
    MultiplexedTensoredU2Box: ClassVar[OpType] = ...
    MultiplexedU2Box: ClassVar[OpType] = ...
    MultiplexorBox: ClassVar[OpType] = ...
    NPhasedX: ClassVar[OpType] = ...
    PauliExpBox: ClassVar[OpType] = ...
    PauliExpCommutingSetBox: ClassVar[OpType] = ...
    PauliExpPairBox: ClassVar[OpType] = ...
    Phase: ClassVar[OpType] = ...
    PhasePolyBox: ClassVar[OpType] = ...
    PhasedISWAP: ClassVar[OpType] = ...
    PhasedX: ClassVar[OpType] = ...
    QControlBox: ClassVar[OpType] = ...
    RangePredicate: ClassVar[OpType] = ...
    Reset: ClassVar[OpType] = ...
    Rx: ClassVar[OpType] = ...
    Ry: ClassVar[OpType] = ...
    Rz: ClassVar[OpType] = ...
    S: ClassVar[OpType] = ...
    SWAP: ClassVar[OpType] = ...
    SX: ClassVar[OpType] = ...
    SXdg: ClassVar[OpType] = ...
    Sdg: ClassVar[OpType] = ...
    SetBits: ClassVar[OpType] = ...
    StatePreparationBox: ClassVar[OpType] = ...
    Stop: ClassVar[OpType] = ...
    Sycamore: ClassVar[OpType] = ...
    T: ClassVar[OpType] = ...
    TK1: ClassVar[OpType] = ...
    TK2: ClassVar[OpType] = ...
    Tdg: ClassVar[OpType] = ...
    ToffoliBox: ClassVar[OpType] = ...
    U1: ClassVar[OpType] = ...
    U2: ClassVar[OpType] = ...
    U3: ClassVar[OpType] = ...
    Unitary1qBox: ClassVar[OpType] = ...
    Unitary2qBox: ClassVar[OpType] = ...
    Unitary3qBox: ClassVar[OpType] = ...
    V: ClassVar[OpType] = ...
    Vdg: ClassVar[OpType] = ...
    WASM: ClassVar[OpType] = ...
    X: ClassVar[OpType] = ...
    XXPhase: ClassVar[OpType] = ...
    XXPhase3: ClassVar[OpType] = ...
    Y: ClassVar[OpType] = ...
    YYPhase: ClassVar[OpType] = ...
    Z: ClassVar[OpType] = ...
    ZZMax: ClassVar[OpType] = ...
    ZZPhase: ClassVar[OpType] = ...
    __entries: ClassVar[dict] = ...
    noop: ClassVar[OpType] = ...
    def __init__(self, value: int) -> None: ...
    @classmethod
    def from_name(cls, arg0: str) -> OpType: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class PauliExpBox(Op):
    def __init__(self, paulis: List[pytket._tket.pauli.Pauli], t: Union[sympy.Expr,float], cx_config_type: CXConfigType = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_cx_config(self) -> CXConfigType: ...
    def get_paulis(self) -> List[pytket._tket.pauli.Pauli]: ...
    def get_phase(self) -> Union[sympy.Expr,float]: ...

class PauliExpCommutingSetBox(Op):
    def __init__(self, pauli_gadgets: List[Tuple[List[pytket._tket.pauli.Pauli],Union[sympy.Expr,float]]], cx_config_type: CXConfigType = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_cx_config(self) -> CXConfigType: ...
    def get_paulis(self) -> List[Tuple[List[pytket._tket.pauli.Pauli],Union[sympy.Expr,float]]]: ...

class PauliExpPairBox(Op):
    def __init__(self, paulis0: List[pytket._tket.pauli.Pauli], t0: Union[sympy.Expr,float], paulis1: List[pytket._tket.pauli.Pauli], t1: Union[sympy.Expr,float], cx_config_type: CXConfigType = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_cx_config(self) -> CXConfigType: ...
    def get_paulis_pair(self) -> Tuple[List[pytket._tket.pauli.Pauli],List[pytket._tket.pauli.Pauli]]: ...
    def get_phase_pair(self) -> Tuple[Union[sympy.Expr,float],Union[sympy.Expr,float]]: ...

class PhasePolyBox(Op):
    @overload
    def __init__(self, n_qubits: int, qubit_indices: Dict[pytket._tket.unit_id.Qubit,int], phase_polynomial: Dict[List[bool],Union[sympy.Expr,float]], linear_transformation: NDArray[numpy.bool_]) -> None: ...
    @overload
    def __init__(self, n_qubits: int, qubit_indices: Dict[pytket._tket.unit_id.Qubit,int], phase_polynomial: List[Tuple[List[bool],Union[sympy.Expr,float]]], linear_transformation: NDArray[numpy.bool_]) -> None: ...
    @overload
    def __init__(self, circuit: Circuit) -> None: ...
    def get_circuit(self) -> Circuit: ...
    @property
    def linear_transformation(self) -> NDArray[numpy.bool_]: ...
    @property
    def n_qubits(self) -> int: ...
    @property
    def phase_polynomial(self) -> Dict[tuple,Union[sympy.Expr,float]]: ...
    @property
    def phase_polynomial_as_list(self) -> List[Tuple[List[bool],Union[sympy.Expr,float]]]: ...
    @property
    def qubit_indices(self) -> Dict[pytket._tket.unit_id.Qubit,int]: ...

class ProjectorAssertionBox(Op):
    def __init__(self, m: NDArray[numpy.complex128], basis: BasisOrder = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_matrix(self) -> NDArray[numpy.complex128]: ...

class QControlBox(Op):
    @overload
    def __init__(self, op: Op, n_controls: int = ..., control_state: List[bool] = ...) -> None: ...
    @overload
    def __init__(self, op: Op, n_controls: int, control_state: int) -> None: ...
    @overload
    def __init__(self, op: Op, n: int = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_control_state(self) -> int: ...
    def get_control_state_bits(self) -> List[bool]: ...
    def get_n_controls(self) -> int: ...
    def get_op(self) -> Op: ...

class RangePredicateOp(ClassicalEvalOp):
    def __init__(self, width: int, upper: int, lower: int) -> None: ...
    @property
    def lower(self) -> int: ...
    @property
    def upper(self) -> int: ...

class SetBitsOp(ClassicalEvalOp):
    def __init__(self, values: List[bool]) -> None: ...
    @property
    def values(self) -> List[bool]: ...

class StabiliserAssertionBox(Op):
    @overload
    def __init__(self, stabilisers: List[pytket._tket.pauli.PauliStabiliser]) -> None: ...
    @overload
    def __init__(self, stabilisers: List[str]) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_stabilisers(self) -> List[pytket._tket.pauli.PauliStabiliser]: ...

class StatePreparationBox(Op):
    def __init__(self, statevector: NDArray[numpy.complex128], is_inverse: bool = ..., with_initial_reset: bool = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_statevector(self) -> NDArray[numpy.complex128]: ...
    def is_inverse(self) -> bool: ...
    def with_initial_reset(self) -> bool: ...

class ToffoliBox(Op):
    @overload
    def __init__(self, permutation: List[Tuple[List[bool],List[bool]]], strat: ToffoliBoxSynthStrat, rotation_axis: OpType = ...) -> None: ...
    @overload
    def __init__(self, permutation: List[Tuple[List[bool],List[bool]]], rotation_axis: OpType = ...) -> None: ...
    @overload
    def __init__(self, n_qubits: int, permutation: List[Tuple[List[bool],List[bool]]], rotation_axis: OpType = ...) -> None: ...
    @overload
    def __init__(self, permutation: Dict[List[bool],List[bool]], strat: ToffoliBoxSynthStrat, rotation_axis: OpType = ...) -> None: ...
    @overload
    def __init__(self, permutation: Dict[List[bool],List[bool]], rotation_axis: OpType = ...) -> None: ...
    @overload
    def __init__(self, n_qubits: int, permutation: Dict[List[bool],List[bool]], rotation_axis: OpType = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_permutation(self) -> Dict[tuple,tuple]: ...
    def get_rotation_axis(self) -> OpType: ...
    def get_strat(self) -> ToffoliBoxSynthStrat: ...

class ToffoliBoxSynthStrat:
    __members__: ClassVar[dict] = ...  # read-only
    Cycle: ClassVar[ToffoliBoxSynthStrat] = ...
    Matching: ClassVar[ToffoliBoxSynthStrat] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class Unitary1qBox(Op):
    def __init__(self, m: NDArray[numpy.complex128]) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_matrix(self) -> NDArray[numpy.complex128]: ...

class Unitary2qBox(Op):
    def __init__(self, m: NDArray[numpy.complex128], basis: BasisOrder = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_matrix(self) -> NDArray[numpy.complex128]: ...

class Unitary3qBox(Op):
    def __init__(self, m: NDArray[numpy.complex128], basis: BasisOrder = ...) -> None: ...
    def get_circuit(self) -> Circuit: ...
    def get_matrix(self) -> NDArray[numpy.complex128]: ...

class WASMOp(ClassicalOp):
    def __init__(self, num_bits: int, num_w: int, n_inputs: List[int], n_outputs: List[int], func_name: str, wasm_uid: str) -> None: ...
    @property
    def func_name(self) -> str: ...
    @property
    def input_widths(self) -> List[int]: ...
    @property
    def n_i32(self) -> int: ...
    @property
    def num_bits(self) -> int: ...
    @property
    def num_w(self) -> int: ...
    @property
    def output_widths(self) -> List[int]: ...
    @property
    def wasm_uid(self) -> str: ...

def fresh_symbol(preferred: str = ...) -> sympy.Symbol: ...
